const extract = (definitions, ...resolvers) => {
  const binded = resolvers.reduce(
    (acc, nextResolver, key) => {
      const fieldName = getFieldName(definitions[key]);
      const typeName = getTypeName(definitions[key]) || acc.lastTypeName;

      const extendFields = acc.resolvers[typeName] || {};
      const resolvers = {
        ...acc.resolvers,
        [typeName]: { ...extendFields, [fieldName]: nextResolver }
      };

      return { resolvers, lastTypeName: typeName }
    },
    { resolvers: {}, lastTypeName: null }
  );

  return {
    resolvers: binded.resolvers || {},
    typeDefs: concatDefinitions(definitions)
  }
};

const getFieldName = definition => {
  const raw = definition.match(/[^\s].+[\s\n:]+$/);
  // @TODO deal when there's no match.
  return (
    raw &&
    raw
      .toString()
      .trim()
      .match(/^(.+):/)
      .pop()
  )
};

const getTypeName = definition => {
  const raw = definition.match(/type\s?.*\s?\{/g);
  // @TODO deal when there's no match.

  return (
    raw &&
    raw
      .pop()
      .match(/[^type\s]\S+[^\s?{^]/)
      .toString()
  )
};

const concatDefinitions = definitions =>
  definitions.join('').replace(/^\s+$(?:\e\n?|\n)/gm, '');

const withAST = toAST => (...args) => {
  const { resolvers, typeDefs } = extract(...args);
  return { resolvers, typeDefs: toAST(typeDefs) }
};

export { concatDefinitions, extract, getFieldName, getTypeName, withAST };
