(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = global || self, factory(global.index = {}));
}(this, function (exports) { 'use strict';

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  function _objectSpread(target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i] != null ? arguments[i] : {};
      var ownKeys = Object.keys(source);

      if (typeof Object.getOwnPropertySymbols === 'function') {
        ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
          return Object.getOwnPropertyDescriptor(source, sym).enumerable;
        }));
      }

      ownKeys.forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    }

    return target;
  }

  var extract = function extract(definitions) {
    for (var _len = arguments.length, resolvers = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      resolvers[_key - 1] = arguments[_key];
    }

    var binded = resolvers.reduce(function (acc, nextResolver, key) {
      var fieldName = getFieldName(definitions[key]);
      var typeName = getTypeName(definitions[key]) || acc.lastTypeName;
      var extendFields = acc.resolvers[typeName] || {};

      var resolvers = _objectSpread({}, acc.resolvers, _defineProperty({}, typeName, _objectSpread({}, extendFields, _defineProperty({}, fieldName, nextResolver))));

      return {
        resolvers: resolvers,
        lastTypeName: typeName
      };
    }, {
      resolvers: {},
      lastTypeName: null
    });
    return {
      resolvers: binded.resolvers || {},
      typeDefs: concatDefinitions(definitions)
    };
  };
  var getFieldName = function getFieldName(definition) {
    var raw = definition.match(/[^\s].+[\s\n:]+$/); // @TODO deal when there's no match.

    return raw && raw.toString().trim().match(/^(.+):/).pop();
  };
  var getTypeName = function getTypeName(definition) {
    var raw = definition.match(/type\s?.*\s?\{/g); // @TODO deal when there's no match.

    return raw && raw.pop().match(/[^type\s]\S+[^\s?{^]/).toString();
  };
  var concatDefinitions = function concatDefinitions(definitions) {
    return definitions.join('').replace(/^\s+$(?:\e\n?|\n)/gm, '');
  };
  var withAST = function withAST(toAST) {
    return function () {
      var _extract = extract.apply(void 0, arguments),
          resolvers = _extract.resolvers,
          typeDefs = _extract.typeDefs;

      return {
        resolvers: resolvers,
        typeDefs: toAST(typeDefs)
      };
    };
  };

  exports.concatDefinitions = concatDefinitions;
  exports.extract = extract;
  exports.getFieldName = getFieldName;
  exports.getTypeName = getTypeName;
  exports.withAST = withAST;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
